%{

#include "flex-actions.h"

%}

/* Obliga a crear una regla explícita para manejar lexemas desconocidos. */
%option nodefault

/* Escanea un único archivo y se detiene. */
%option noyywrap

/* Dispone una variable global "yylineno" con el número de línea actual. */
%option yylineno

/* Patrones reutilizables. */
crlf			\r\n
digit			[0-9]
char            [a-zA-Z]
decimal         [.]
endline			\n
/* whitespace		[\f\n\r\t\v\0 ] */
whitespace      [ \f\n\r\t\v]

%%

"+"											{ return ADD; }
"-"											{ return SUB; }
"*"											{ return MUL; }
"/"											{ return DIV; }
"^"											{ return POW; }
"fun"                                       { return FUNCTION; }
"function"                                  { return FUNCTION; }
"if"                                        { return IF; }
"else"                                      { return ELSE; }
"for"                                       { return FOR; }
"while"                                     { return WHILE; }
"print"                                     { return PRINT; }
";"                                         { return SEMICOLON; }
"o"                                         { return COMPOSITE; }
"'"                                         { return SQUOTE; }
"derivative"                                { return DERIVATIVE; }
"integrate"                                 { return INTEGRATE; }
"int"                                       { return INTEGRATE; }
"between"                                   { return BETWEEN; }
"and"                                       { return AND_INT; }
"~"                                         { return AND_INT; }
"evaluate"                                  { return EVALUATE; }
"eval"                                      { return EVALUATE; }
","                                         { return COMMA; }
":"                                         { return COLON; }
"<"                                         { return LESSER; }
"<="                                        { return LESSER_EQUAL; }
">"                                         { return GREATER; }
">="                                        { return GREATER_EQUAL; }
"in"                                        { return IN; }
"{"                                         { return OPEN_BRACES; }
"}"                                         { return CLOSE_BRACES; }
"["                                         { return OPEN_BRACKETS; }
"]"                                         { return CLOSE_BRACKETS; }
"("											{ return OPEN_PARENTHESIS; }
")"											{ return CLOSE_PARENTHESIS; }
"="                                         { return ASSIGN; }
"=="                                        { return EQUAL; }
"&&"                                        { return AND; }
"||"                                        { return OR; }
"!"                                         { return NOT; }
"+="                                        { return ADD_ASSIGN; }
"++"                                        { return INCREMENT; }
"-="                                        { return SUB_ASSIGN; }
"--"                                        { return DECREMENT; }
"*="                                        { return MUL_ASSIGN; }
"/="                                        { return DIV_ASSIGN; }
"\""                                        { return DQUOTE; }
"var"                                       { return VAR; }
"err"                                       { return ERR; }
"taylor"                                    { return TAYLOR; }


{digit}+{decimal}{digit}+                   { return DoublePatternAction(yytext); }

{digit}+						     		{ return IntegerPatternAction(yytext); }

{char}+                                     { return StringPatternAction(yytext); }

\"{char}+\"                                 { return EscapeStringPatternAction(yytext); }

{whitespace}								{ IgnoredPatternAction(yytext); }

.											{ return UnknownPatternAction(yytext); }

%%
