%{

#include "bison-actions.h"

%}

// IDs de los tokens generados desde Flex:

%token ADD
%token SUB
%token MUL
%token DIV
%token POW
%token FUNCTION
%token IF
%token ELSE
%token FOR
%token WHILE
%token PRINT
%token SEMICOLON
%token COMPOSITE
%token SQUOTE
%token DERIVATIVE
%token INTEGRATE
%token BETWEEN
%token AND_INT
%token EVALUATE
%token IN
%token OPEN_BRACES
%token OPEN_BRACKETS
%token CLOSE_BRACES
%token CLOSE_BRACKETS
%token OPEN_PARENTHESIS
%token CLOSE_PARENTHESIS
%token ASSIGN
%token EQUAL
%token AND
%token OR
%token NOT
%token ADD_ASSIGN
%token INCREMENT
%token SUB_ASSIGN
%token DECREMENT
%token MUL_ASSIGN
%token DIV_ASSIGN
%token DQUOTE
%token COMMA
%token COLON
%token LESSER
%token LESSER_EQUAL
%token GREATER
%token GREATER_EQUAL
%token COMPARE
%token VAR
%token ERR
%token TAYLOR
%token ESCAPESTRING
%token DOUBLE
%token INTEGER
%token STRING

// Reglas de asociatividad y precedencia (de menor a mayor):
%left ADD SUB
%left MUL DIV
%left POW

%%

program: statements												//	{ $$ = ProgramGrammarAction($1); }
	;

expression: constant
	| STRING
	| expression ADD expression						 			//	{ $$ = AdditionExpressionGrammarAction($1, $3); }
	| expression SUB expression									//	{ $$ = SubtractionExpressionGrammarAction($1, $3); }
	| expression MUL expression									//	{ $$ = MultiplicationExpressionGrammarAction($1, $3); }
	| expression DIV expression									//	{ $$ = DivisionExpressionGrammarAction($1, $3); }
	| expression POW expression									//	{ $$ = PowerExpressionGrammarAction($1, $3); }
	| OPEN_PARENTHESIS expression CLOSE_PARENTHESIS
	| SUB expression
	| number
	;

domain: STRING LESSER number									//	{ $$ = LesserExpressionGrammarAction($1, $3); }
	| STRING LESSER_EQUAL number								//	{ $$ = LesserEqualExpressionGrammarAction($1, $3); }
	| STRING GREATER number										//	{ $$ = GreaterExpressionGrammarAction($1, $3); }
	| STRING GREATER_EQUAL number								//	{ $$ = GreaterEqualExpressionGrammarAction($1, $3); }
	| STRING EQUAL number										//	{ $$ = EqualExpressionGrammarAction($1, $3); }
	| number LESSER STRING										//	{ $$ = LesserExpressionGrammarAction($1, $3); }
	| number LESSER_EQUAL STRING								//	{ $$ = LesserEqualExpressionGrammarAction($1, $3); }
	| number GREATER STRING										//	{ $$ = GreaterExpressionGrammarAction($1, $3); }
	| number GREATER_EQUAL STRING								//	{ $$ = GreaterEqualExpressionGrammarAction($1, $3); }
	| number EQUAL STRING										//	{ $$ = EqualExpressionGrammarAction($1, $3); }
	| number LESSER STRING LESSER number
	| number LESSER_EQUAL STRING LESSER number
	| number LESSER STRING LESSER_EQUAL number
	| number LESSER_EQUAL STRING LESSER_EQUAL number 
	;

boolean: number LESSER number
	| number LESSER_EQUAL number
	| number GREATER number
	| number GREATER_EQUAL number
	| number EQUAL number
	| boolean OR boolean
	| boolean AND boolean
	| NOT boolean
	;

constant: DOUBLE
	| INTEGER
	| SUB constant
	;

declare_function: FUNCTION STRING ASSIGN function
	| FUNCTION STRING ASSIGN OPEN_BRACES functions CLOSE_BRACES
	;

function: expression
	| expression COLON domain
	| composite
	| taylor
	;

functions: function
	| function COMMA functions
	;

composite: STRING COMPOSITE STRING
	| composite COMPOSITE STRING
	;

numbers: number
	| number COMMA numbers
	;

strings: STRING
	| STRING COMMA strings

numArray: OPEN_BRACKETS numbers CLOSE_BRACKETS
	;

stringArray: OPEN_BRACKETS strings CLOSE_BRACKETS
	;

evaluate: EVALUATE STRING IN number							//{ $$ = IntegerConstantGrammarAction($1); }
	| EVALUATE STRING IN numArray								//{ $$ = IntegerConstantGrammarAction($1); }
	| STRING OPEN_PARENTHESIS number CLOSE_PARENTHESIS		//{ $$ = IntegerConstantGrammarAction($1); }
	| EVALUATE stringArray IN number							//{ $$ = IntegerConstantGrammarAction($1); }
	| EVALUATE stringArray IN numArray								//{ $$ = IntegerConstantGrammarAction($1); }
	| stringArray OPEN_PARENTHESIS number CLOSE_PARENTHESIS		//{ $$ = IntegerConstantGrammarAction($1); }
	;

integrate: INTEGRATE STRING BETWEEN number AND_INT number
	| INTEGRATE STRING number AND_INT number
	| INTEGRATE stringArray BETWEEN number AND_INT number
	| INTEGRATE stringArray number AND_INT number
	| integrate ERR constant
	;

squotes: SQUOTE
	| SQUOTE squotes
	; 

statement: evaluate SEMICOLON
	| integrate SEMICOLON
	| derivative SEMICOLON
	| declare_function SEMICOLON
	| if_statement
	| ifelse_statement
	| for_statement
	| while_statement
	| print SEMICOLON
	| declare_variable SEMICOLON
	| assign_variable SEMICOLON
	;

print: PRINT OPEN_PARENTHESIS number CLOSE_PARENTHESIS
	| PRINT OPEN_PARENTHESIS STRING CLOSE_PARENTHESIS
	| PRINT OPEN_PARENTHESIS escapedString CLOSE_PARENTHESIS
	;

escapedString: ESCAPESTRING
	| ESCAPESTRING ADD escapedString
	;

statements: statement
	| statement statements
	;

derivative: DERIVATIVE STRING IN constant
	| DERIVATIVE INTEGER STRING IN constant
	| STRING squotes OPEN_PARENTHESIS constant CLOSE_PARENTHESIS
	| DERIVATIVE stringArray IN constant
	| DERIVATIVE INTEGER stringArray IN constant
	| stringArray squotes OPEN_PARENTHESIS constant CLOSE_PARENTHESIS
	| derivative ERR constant
	;

if_statement: IF OPEN_PARENTHESIS boolean CLOSE_PARENTHESIS OPEN_BRACKETS statements CLOSE_BRACKETS
	| IF OPEN_PARENTHESIS boolean CLOSE_PARENTHESIS statement
	;

ifelse_statement: if_statement ELSE OPEN_BRACKETS statements CLOSE_BRACKETS
	| ELSE statement
	;

for_statement: FOR OPEN_PARENTHESIS declare_variable SEMICOLON boolean SEMICOLON assign_variable CLOSE_PARENTHESIS OPEN_BRACKETS statements CLOSE_BRACKETS
	| FOR OPEN_PARENTHESIS SEMICOLON boolean SEMICOLON assign_variable CLOSE_PARENTHESIS OPEN_BRACKETS statements CLOSE_BRACKETS
	| FOR OPEN_PARENTHESIS declare_variable SEMICOLON boolean SEMICOLON assign_variable CLOSE_PARENTHESIS statement
	| FOR OPEN_PARENTHESIS SEMICOLON boolean SEMICOLON assign_variable CLOSE_PARENTHESIS statement
	;

taylor: TAYLOR INTEGER STRING
	;

while_statement: WHILE OPEN_PARENTHESIS boolean CLOSE_PARENTHESIS OPEN_BRACKETS statements CLOSE_BRACKETS
	| WHILE OPEN_PARENTHESIS boolean CLOSE_PARENTHESIS statement
	;

declare_variable: VAR STRING
	| VAR STRING ASSIGN number
	| VAR STRING ASSIGN expression
	; 

number: evaluate
	| integrate
	| derivative
	| constant
	| number ADD number
	| number DIV number
	| number SUB number
	| number POW number
	| number MUL number
	| SUB number
	| OPEN_PARENTHESIS number CLOSE_PARENTHESIS
	;

assign_variable: STRING ASSIGN number
	| STRING ADD_ASSIGN number
	| STRING SUB_ASSIGN number
	| STRING MUL_ASSIGN number
	| STRING DIV_ASSIGN number
	| STRING INCREMENT
	| STRING DECREMENT
	| STRING ASSIGN expression
	; 
	
%%
